@@ -45,11 +45,7 @@ tvCov <- function(x, bw, est = c("lc", "ll"), tkernel = c("Epa", "Gaussian"))
   obs <- NROW(x)
   neq <- NCOL(x)
   tkernel <- match.arg(tkernel)
-  if(!(tkernel %in% c("Epa", "Gaussian")))
-    tkernel <- "Epa"
   est <- match.arg(est)
-  if(!(est %in% c("lc", "ll")))
-    est <- "lc"
   Sigma <- array(0, dim = c(neq, neq, obs))
   resid.2 <- numeric(obs)
   if(length(bw) > 1)
@@ -57,26 +53,26 @@ tvCov <- function(x, bw, est = c("lc", "ll"), tkernel = c("Epa", "Gaussian"))
   time.grid <- 1:obs
   for (t in 1:obs)
   {
-    x2 <- (time.grid - t)/obs
-    kernel.bw <- .kernel(x2, bw, tkernel = tkernel)/bw
+    tau0 <- (time.grid - t)/obs
+    kernel.bw <- .kernel(tau0, bw, tkernel = tkernel)/bw
     w0 <- sum(kernel.bw)
     r.2 <- matrix(0, neq, neq)
     if(est == "lc")
     {
       for (index in which(kernel.bw != 0))
       {
-        r.2 <- crossprod(t(x[index,])) * kernel.bw[index]/w0 + r.2
+        r.2 <- tcrossprod(x[index,]) * kernel.bw[index]/w0 + r.2
       }
     }
     else if (est == "ll")
     {
-      w1 <- sum(kernel.bw * x2)
-      w2 <- sum(kernel.bw * (x2)^2)
+      w1 <- sum(kernel.bw * tau0)
+      w2 <- sum(kernel.bw * (tau0)^2)
       den.c <- w0 * w2 - w1^2
       if (den.c == 0) return(.Machine$double.xmax)
       for (index in which(kernel.bw != 0))
       {
-        r.2 <- crossprod(t(x[index,])) * kernel.bw[index] * (w2-w1 * x2[index])/den.c + r.2
+        r.2 <- tcrossprod(x[index,]) * kernel.bw[index] * (w2-w1 * tau0[index])/den.c + r.2
       }
     }
     Sigma[,,t] <- r.2
@@ -93,17 +89,13 @@ tvCov <- function(x, bw, est = c("lc", "ll"), tkernel = c("Epa", "Gaussian"))
 #' @return A scalar with the mean squared error.
 #' @keywords internal
 #'
-.tvCov.cv <- function(bw, x, est = c("lc", "ll"), tkernel = c("Epa", "Gaussian"))
+.tvCov.cv <- function(bw, x, cv.block = 0, est = c("lc", "ll"), tkernel = c("Epa", "Gaussian"))
 {
   x <- as.matrix(x)
   obs <- NROW(x)
   neq <- NCOL(x)
   tkernel <- match.arg(tkernel)
   est <- match.arg(est)
-  if(!(tkernel %in% c("Epa", "Gaussian")))
-    tkernel <- "Epa"
-  if(!(est %in% c("lc", "ll")))
-    est <- "lc"
   Sigma <-  array(0, dim = c(neq, neq, obs))
   resid.2 <- numeric(obs)
   if(length(bw)>1) 
@@ -111,31 +103,34 @@ tvCov <- function(x, bw, est = c("lc", "ll"), tkernel = c("Epa", "Gaussian"))
   time.grid <- 1:obs
   for (t in 1:obs)
   {
-    x2 <- (time.grid-t)/obs
-    kernel.bw <- .kernel(x2, bw, tkernel = tkernel)/bw
-    kernel.bw[t] <- 0
+    tau0 <- (time.grid-t)/obs
+    kernel.bw <- .kernel(tau0, bw, tkernel = tkernel)/bw
+    kernel.bw[max(1, t-cv.block):min(t+cv.block, obs)] <- 0
     w0 <- sum(kernel.bw)
+    k.index <- which(kernel.bw != 0)
+    if (sum(k.index != 0) < 3)
+      return (.Machine$double.xmax)
     r.2 <- matrix(0, neq, neq)
     if(est == "lc")
     {
-      for (index in which(kernel.bw != 0))
+      for (index in k.index)
       {
-        r.2 <- crossprod(t(x[index,])) * kernel.bw[index]/w0 + r.2
+        r.2 <- tcrossprod(x[index,]) * kernel.bw[index]/w0 + r.2
       }
     }
     else if (est == "ll")
     {
-      w1 <- sum(kernel.bw * x2)
-      w2 <- sum(kernel.bw * (x2)^2)
+      w1 <- sum(kernel.bw * tau0)
+      w2 <- sum(kernel.bw * (tau0)^2)
       den.c <- w0 * w2 - w1^2
       if (den.c == 0) return(.Machine$double.xmax)
-      for (index in which(kernel.bw != 0))
+      for (index in k.index)
       {
-        r.2 <- crossprod(t(x[index,])) * kernel.bw[index] * (w2-w1 * x2[index])/den.c + r.2
+        r.2 <- tcrossprod(x[index,]) * kernel.bw[index] * (w2 - w1 * tau0[index])/den.c + r.2
       }
     }
     Sigma[,,t] <- r.2
-    resid.2[t] <- sum((crossprod(t(x[t,])) - Sigma[,,t])^2)
+    resid.2[t] <- sum((tcrossprod(x[t,]) - Sigma[,,t])^2)
   }
   return(mean(resid.2))
 }
