@@ -36,16 +36,12 @@
 #' \item{n.ahead}{Number of ahead impulse response functions.}
 #' \item{ortho}{Logical, orthogonal or not impuluse response function.}
 #' \item{ortho.cov}{Character, either 'const' or 'tv' (default). This 
-#' parameter is used when the orthogonal tv-IRF is calculated. The default 
+#' parameter is used when the orthogonal TV-IRF is calculated. The default 
 #' is using an error time-varying variance-covariance.}
 #' \item{bw.cov}{A scalar with the bandwidth to estimate the errors 
 #' variance-covariance matrix. If NULL, it is calculated by cross-validation.}
 #' \item{cumulative}{Logical, if TRUE the cumulated impulse response 
 #' coefficients are computed. Default is FALSE.}
-#' \item{level}{Numeric, confidence interval range. The default is zero.}
-#' \item{runs}{Number of bootstrap replications.}
-#' \item{tboot}{Type of bootstrap.}
-#' \item{BOOT}{List with all bootstrap replications of \code{tvirf}, if done.}
 #' 
 #' @seealso \code{\link{bw}}, \code{\link{tvVAR}}, \code{\link{confint}}, 
 #' \code{\link{plot}}, \code{\link{print}} and \code{\link{summary}}
@@ -64,23 +60,16 @@
 #' model.tvIRF2 <- tvIRF(model.tvVAR, cumulative = TRUE)
 #' }
 #' 
-#' @export
-tvIRF<-function (x, ...)
-{
-  UseMethod("tvIRF", x)
-}
-
 #' @rdname tvIRF
-#' @method tvIRF tvvar
 #' @export
-tvIRF.tvvar<-function (x, impulse = NULL, response = NULL, n.ahead = 10,
+tvIRF<-function (x, impulse = NULL, response = NULL, n.ahead = 10,
                        ortho = TRUE,  ortho.cov = c("tv", "const"), 
                        bw.cov = NULL, cumulative = FALSE, ...)
 {
   if (!(class(x) == "tvvar")) 
     stop("\nPlease provide an object of class 'tvvar', 
          generated by 'tvVAR()'.\n")
-  y.names <- colnames(x$y)
+  y.names <- colnames(x$y.orig)
   if (is.null(impulse))
   {
     impulse <- y.names
@@ -117,7 +106,7 @@ tvIRF.tvvar<-function (x, impulse = NULL, response = NULL, n.ahead = 10,
                  impulse = impulse, x = x,  n.ahead = n.ahead, ortho = ortho,
                  ortho.cov = ortho.cov, bw.cov = irs$bw.cov, 
                  cumulative = cumulative, level = 0, runs = 0, 
-                 tboot = NULL, BOOT = NULL, call = match.call())
+                 tboot = NULL, BOOT = NULL)
   class(result) <- "tvirf"
   return(result)
 }
